@@ -1,63 +1,59 @@
 package Hook::Output::Tiny;
-
-use 5.006;
 use strict;
 use warnings;
 
-our $VERSION = '0.04';
+our $VERSION = '0.05';
 
 sub new {
-    return bless {
-        stdout => {_struct()},
-        stderr => {_struct()},
-    }, shift;
+    my %struct = map { $_ => {_struct()} } qw(stderr stdout);
+    return bless \%struct, shift;
 }
 sub hook {
     my ($self, $handle) = @_;
 
-    my @handles = $self->_handles($handle);
-
-    if (grep {$_ eq 'stderr'} @handles){
-        $self->_stderr;
-    }
-    if (grep {$_ eq 'stdout'} @handles){
-        $self->_stdout;
-        select $self->{stdout}{handle};
+    for(_handles($handle)) {
+        if ($_ eq 'stderr') {
+            $self->_stderr;
+        }
+        else {
+            $self->_stdout;
+        }
     }
 }
 sub unhook {
     my ($self, $handle) = @_;
 
-    my @handles = $self->_handles($handle);
+    my @handles = _handles($handle);
 
     if (grep {$_ eq 'stdout'} @handles) {
-        select STDOUT or die $!;
-        $self->{stdout}{state} = 0;
+        close STDOUT;
+        open STDOUT, ">&$self->{stdout}{handle}" or die $!;
     }
     if (grep {$_ eq 'stderr'} @handles) {
-        $self->{stderr}{state} = 0;
         close STDERR;
         open STDERR, ">&$self->{stderr}{handle}" or die $!;
     }
 }
 sub stdout {
-    my $self = shift;
-    return split /\n/, $self->{stdout}{data};
+    return split /\n/, $_[0]->{stdout}{data};
 }
 sub stderr {
-    my $self = shift;
-    return split /\n/, $self->{stderr}{data};
+    return split /\n/, $_[0]->{stderr}{data};
 }
 sub flush {
     my ($self, $handle) = @_;
-    my @handles = $self->_handles($handle);
+    my @handles = _handles($handle);
     for (@handles){
         delete $self->{$_}{data};
     }
 }
 sub write {
     my ($self, $fn, $handle) = @_;
-    my @handles = $self->_handles($handle);
+    if ($fn eq 'stderr' || $fn eq 'stdout'){
+        die "write() requires a file name sent in before the handle\n";
+    }
+
+    my @handles = _handles($handle);
     for (@handles){
         open my $wfh, '>>', $fn or die $!;
         print $wfh $self->{$_}{data};
@@ -67,81 +63,82 @@ sub write {
 }
 sub _stdout {
     my $self = shift;
-    open $self->{stdout}{handle}, '>>', \$self->{stdout}{data}
-      or die "can't hook STDOUT: $!";
-    $self->{stdout}{state} = 1;
+    open $self->{stdout}{handle}, ">&STDOUT" or die "can't hook STDOUT: $!";
+    close STDOUT;
+    open STDOUT, '>>', \$self->{stdout}{data} or die $!;
 }
 sub _stderr {
     my $self = shift;
-    open $self->{stderr}{handle}, ">&STDERR"
-      or die "can't hook STDERR: $!";
+    open $self->{stderr}{handle}, ">&STDERR" or die "can't hook STDERR: $!";
     close STDERR;
     open STDERR, '>>', \$self->{stderr}{data} or die $!;
 }
 sub _struct {
-     return (
-        state => 0,
-        handle => *fh,
-        data => '',
-    );
+     return (handle => *fh, data => '');
 }
 sub _handles {
-    my ($self, $handle) = @_;
-    my @handles;
-    if ($handle){
-        push @handles, $handle;
-    }
-    else {
-        push @handles, 'stdout', 'stderr';
-    }
-    return @handles;
+    my $handle = shift;
+    my $sub = (caller(1))[3];
+    _check_param($sub, $handle) if $handle;
+    return $handle ? ($handle) : qw(stderr stdout);
 }
+sub _check_param {
+    # validates the $handle param
+    my ($sub, $handle) = @_;
+    if (! grep {$handle eq $_} qw(stderr stdout)){
+        die "$sub() either takes 'stderr', 'stdout' or no params\n" .
+            "You supplied '$handle'\n";
+    }
+}
+1;
+
 =head1 NAME
 
 Hook::Output::Tiny - Easily enable/disable trapping of STDOUT/STDERR
 
 =for html
-<a href="http://travis-ci.org/stevieb9/p5-hook-output-tiny"><img src="https://secure.travis-ci.org/stevieb9/p5-hook-output-tiny.png"/>
+<a href="http://travis-ci.org/stevieb9/p5-hook-output-tiny"><img src="https://secure.travis-ci.org/stevieb9/p5-hook-output-tiny.png"/></a>
+<a href="https://ci.appveyor.com/project/stevieb9/p5-hook-output-tiny"><img src="https://ci.appveyor.com/api/projects/status/br01o72b3if3plsw/branch/master?svg=true"/></a>
 <a href='https://coveralls.io/github/stevieb9/p5-hook-output-tiny?branch=master'><img src='https://coveralls.io/repos/stevieb9/p5-hook-output-tiny/badge.svg?branch=master&service=github' alt='Coverage Status' /></a>
 
 =head1 SYNOPSIS
 
     use Hook::Output::Tiny;
 
-    my $h = Hook::Output::Tiny->new;
+    my $output = Hook::Output::Tiny->new;
 
     # trap either
 
-    $h->hook('stdout');
-    my @out = $h->stdout;
+    $output->hook('stdout');
+    my @out = $output->stdout;
 
-    $h->hook('stderr');
-    my @err = $h->stderr;
+    $output->hook('stderr');
+    my @err = $output->stderr;
 
     # untrap either
 
-    $h->unhook('stdout');
-    $h->unhook('stderr');
+    $output->unhook('stdout');
+    $output->unhook('stderr');
 
     # trap/untrap both simultaneously
 
-    $h->hook;
-    $h->unhook;
+    $output->hook;
+    $output->unhook;
 
     # delete all entries from both (can specify individually)
 
-    $h->flush;
+    $output->flush;
 
     # append to a file (can specify individually)
 
-    $h->write('file.txt');
+    $output->write('file.txt');
 
 =head1 DESCRIPTION
 
 Extremely lightweight mechanism for trapping C<STDOUT>, C<STDERR> or both.
 
 We save the captured output internally, so on long running applications, memory
-usage may become an issue if you don't flush out or write out the data.
+usage may become an issue if you don't C<flush()> out or C<write()> out the data.
 
 There are many modules that perform this task. I wrote this one for fun, and to
 be as small and as simple as possible.
@@ -194,20 +191,42 @@ Testing scenario...
     use Test::More;
 
     my $output = Hook::Output::Tiny->new;
+    my $thing = Foo::Bar->new;
 
     $output->hook;
 
-    my $thing = Foo::Bar->new;
+    $thing->do;
 
     $output->unhook;
 
     is ($thing->do(), 1, "thing() ok");
-    is ($output->stdout, 1, "got expected STDOUT");
+    is ($output->stdout, 2, "got expected STDOUT");
     is ($output->stderr, 0, "got no STDERR");
 
-    for ($output->stdout){
-        like ($_, 'Foo::Bar object initialized', "STDOUT ok");
-    }
+    my @stdout = $output->stdout;
+
+    like ($stdout[0], qr/do() called/, "STDOUT ok");
+    is ($stdout[1], 'did', "STDOUT said do() 'did'");
+
+    $output->hook;
+
+    $thing->error;
+
+    $output->unhook;
+
+    @stderr = $output->stderr;
+
+    like ($stderr[0], qr/error/, "error() errored properly");
+
+=head1 SEE ALSO
+
+L<Capture::Tiny>, the de-facto in-core standard.
+
+L<IO::CaptureOutput>
+
+L<IO::Capture::Output>
+
+L<Hook::Output::File>
 
 =head1 AUTHOR
 
